//#define UNICODE
#include <windows.h>
#include <gdiplus.h>
#include <winuser.h>
#include <stdio.h> 
#include <tchar.h>
#include <stdlib.h> //these two files are for the random number generator
#include <time.h>
#include "resource.h"

#define APPNAME "GeneteX"
#define PASSSCREENX 110
#define PASSSCREENY 0

using namespace Gdiplus;


POINT mousepoint;

//function declarations
void DisplayPassword(HDC hdc);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
unsigned char *VerifyPass(unsigned char password[12]);
unsigned char *GeneratePass(unsigned char decodeddata[5]);
void GenerateRandomPass();
void InitializeBitmaps();
void DestroyBitmaps();
Bitmap *CreatePasswordBitmap(unsigned char *password);
void GenerateAndDisplayPasswordFromCheckBoxes(HWND hWnd);
void SetCheckBoxesFromPassword(HWND hWnd);
void EnableCheckBoxes(HWND hWnd,BOOL bEnable);

//Globals
//This was taken from the rom. Its a table used to scramble the password up
unsigned char passwordcrambledata[96] = {
      06,01,00,03,07,02,05,04,
      05,04,03,00,06,07,02,01,
      06,04,05,00,03,02,01,07,
      00,02,07,04,06,05,03,01,
      01,02,04,06,03,05,00,07,
      07,01,03,06,00,02,05,04,
      03,04,00,07,06,02,01,05,
      03,07,04,06,00,01,02,05,
      02,06,04,01,00,07,05,03,
      01,04,00,03,07,06,02,05,
      07,06,01,05,03,00,04,02,
	  03,06,00,02,07,01,05,04
};

//This table here tells what bits of data to encode in each digit of the password
unsigned char bittable[60] = {
      0x20,0x01,0x80,0x00,0x04,0x00,0x02,0x00,0x08,0x10,0x40,0x00,
      0x01,0x00,0x10,0x04,0x00,0x20,0x00,0x08,0x02,0x00,0x80,0x40,
      0x00,0x08,0x00,0x10,0x01,0x40,0x04,0x20,0x00,0x02,0x00,0x80,
      0x00,0x04,0x20,0x80,0x00,0x08,0x00,0x10,0x00,0x40,0x02,0x01,
      0x08,0x00,0x00,0x00,0x02,0x00,0x01,0x00,0x04,0x00,0x00,0x00
};

HINSTANCE g_hInstance; //global hInstance variable
unsigned char g_currentdisplayedpass[12]; //this stores the current password that is onscreen
Bitmap *g_blankscreenbitmap = NULL; //global blank password screen bitmap
Bitmap *g_numberbitmaps[8]; //pointer to bitmaps for each number

//This function draws the password image to the screen
void DisplayPassword(HDC hdc)
{
   Graphics   graphics(hdc);
   Bitmap     *tempbitmap = CreatePasswordBitmap(g_currentdisplayedpass);
   graphics.DrawImage(tempbitmap,PASSSCREENX,PASSSCREENY,tempbitmap->GetWidth(),tempbitmap->GetHeight());

   delete tempbitmap;
}

INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, PSTR, INT iCmdShow)
{
   HWND                hwnd;
   MSG                 msg;
   WNDCLASS            wndclass;
   GdiplusStartupInput gdiplusStartupInput;
   ULONG_PTR           gdiplusToken;

   // Set the Global hInstance variable
   g_hInstance = hInstance;

   // Initialize GDI+.
   GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
   
   // Initialize Random Number Generator
   srand( (unsigned)time( NULL ) );
   
   //Initialize the password bitmaps
   InitializeBitmaps();

   wndclass.style          = CS_HREDRAW | CS_VREDRAW;
   wndclass.lpfnWndProc    = WndProc;
   wndclass.cbClsExtra     = 0;
   wndclass.cbWndExtra     = DLGWINDOWEXTRA; //this is essential since its a dialog box window
   wndclass.hInstance      = hInstance;
   wndclass.hIcon          = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
   wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
   wndclass.hbrBackground  = (HBRUSH)COLOR_BACKGROUND;
   wndclass.lpszMenuName   = NULL;
   wndclass.lpszClassName  = APPNAME;

   if (!RegisterClass (&wndclass)){
      MessageBox (NULL, TEXT ("Error Registering window class!"),
                  APPNAME, MB_ICONERROR) ;
      return 0;
   }

   hwnd = CreateDialog (hInstance, MAKEINTRESOURCE(IDD_MAINFORM), 0, NULL) ;

   if(hwnd != NULL){
       ShowWindow(hwnd, iCmdShow);
   }
   else{
      MessageBox(hwnd, "CreateDialog returned NULL", "Warning!",  
           MB_OK | MB_ICONINFORMATION);
      return 0;
   }

   CheckDlgButton(hwnd,IDC_BIT28CHECK,BST_CHECKED); //Start at intro level bit should be on by defualt
   GenerateAndDisplayPasswordFromCheckBoxes(hwnd); //Show the password

   UpdateWindow(hwnd);

   while(GetMessage(&msg, NULL, 0, 0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }

   DestroyBitmaps(); //Destroy our password bitmaps before shutting down GDI+
   GdiplusShutdown(gdiplusToken);

   return 0;
//   return (int)msg.wParam;
}  // WinMain

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, 
   WPARAM wParam, LPARAM lParam)
{
   HDC          hdc;
   PAINTSTRUCT  ps;

   switch(message)
   {
   case WM_PAINT:  //display the password on WM_PAINT
      hdc = BeginPaint(hWnd, &ps);
	  DisplayPassword(hdc);
      EndPaint(hWnd, &ps);
      return 0;

   case WM_LBUTTONDOWN: //this code detects what digit you clicked on and increments it
      mousepoint.x = MAKEPOINTS(lParam).x;
	  mousepoint.y = MAKEPOINTS(lParam).y;

	  if((mousepoint.y > 16+PASSSCREENY) && (mousepoint.y < 156+PASSSCREENY) &&
		  (mousepoint.x > 33+PASSSCREENX) && (mousepoint.x < 222+PASSSCREENX)){ //this is for clicking on a password number
		  int x = (mousepoint.x-(33+PASSSCREENX))/48;
		  int y = (mousepoint.y-(16+PASSSCREENY))/48;
		  if(y*4+x < 12)g_currentdisplayedpass[y*4+x]++;
		  g_currentdisplayedpass[y*4+x] = g_currentdisplayedpass[y*4+x]%8;
		  hdc = GetDC(hWnd);
		  DisplayPassword(hdc);
		  ReleaseDC(hWnd,hdc);
	  }
	  return 0;

   case WM_RBUTTONDOWN: //same as above but the right button decrements it instead
      mousepoint.x = MAKEPOINTS(lParam).x;
	  mousepoint.y = MAKEPOINTS(lParam).y;

	  if((mousepoint.y > 16+PASSSCREENY) && (mousepoint.y < 156+PASSSCREENY) &&
		  (mousepoint.x > 33+PASSSCREENX) && (mousepoint.x < 222+PASSSCREENX)){ //this is for clicking on a password number
		  int x = (mousepoint.x-(33+PASSSCREENX))/48;
		  int y = (mousepoint.y-(16+PASSSCREENY))/48;
		  if(y*4+x < 12)g_currentdisplayedpass[y*4+x]--;
		  g_currentdisplayedpass[y*4+x] = g_currentdisplayedpass[y*4+x]%8;
		  hdc = GetDC(hWnd);
		  DisplayPassword(hdc);
		  ReleaseDC(hWnd,hdc);
	  }
	  return 0;

   case WM_COMMAND :
      switch (LOWORD (wParam))
	  {
	     case IDC_DECODEPASS :
			 SetCheckBoxesFromPassword(hWnd);
			 return 0;

		 case IDC_BIT1CHECK :
		 case IDC_BIT2CHECK :
		 case IDC_BIT3CHECK :
		 case IDC_BIT4CHECK :
		 case IDC_BIT5CHECK :
		 case IDC_BIT6CHECK :
		 case IDC_BIT7CHECK :
		 case IDC_BIT8CHECK :
		 case IDC_BIT9CHECK :
		 case IDC_BIT10CHECK :
		 case IDC_BIT11CHECK :
		 case IDC_BIT12CHECK :
		 case IDC_BIT13CHECK :
		 case IDC_BIT14CHECK :
		 case IDC_BIT15CHECK :
		 case IDC_BIT16CHECK :
		 case IDC_BIT17CHECK :
		 case IDC_BIT18CHECK :
		 case IDC_BIT19CHECK :
		 case IDC_BIT20CHECK :
		 case IDC_BIT21CHECK :
		 case IDC_BIT22CHECK :
		 case IDC_BIT23CHECK :
		 case IDC_BIT24CHECK :
		 case IDC_BIT25CHECK :
		 case IDC_BIT26CHECK :
		 case IDC_BIT27CHECK :
		 case IDC_BIT29CHECK :
			GenerateAndDisplayPasswordFromCheckBoxes(hWnd); //we must update our password if a check was clicked
			return 0;

			//We need a special case for bit 28 because if it isn't set then bits 0-24
			//must be disabled.
		 case IDC_BIT28CHECK :
			 if(IsDlgButtonChecked(hWnd,IDC_BIT28CHECK) == BST_UNCHECKED)EnableCheckBoxes(hWnd,FALSE);
			 else EnableCheckBoxes(hWnd,TRUE);
			 GenerateAndDisplayPasswordFromCheckBoxes(hWnd);
			 return 0;
      }

   case WM_DESTROY:
      PostQuitMessage(0);
      return 0;

   default:
      return DefWindowProc(hWnd, message, wParam, lParam);
   }
} // WndProc

//returns a pointer to decoded password data on success, otherwise returns NULL on error
unsigned char *VerifyPass(unsigned char *password){
	int x,y,i;
	unsigned char descrambledpass[12];
	static unsigned char decodeddata[5];

	for(x = 11;x >= 0;x--){
	//This first set of code is based on assembly found in megaman x version 1.1 at $00:F2E9
		for(y = 7;y >= 0;y--){
			if(passwordcrambledata[(x*8)+y] == password[x])break;
		}
		descrambledpass[x] = y;
	}

	//So now we have the password descrambled, but it still hasn't been decoded and verified
	//This code which is based on $00:F18C in Megaman X 1.1 should do that.
	for(i = 0;i < 6;i++)decodeddata[i] = 0; //this clears decodeddata

	for(x = 11;x >= 0;x--){
		y = 0;
		unsigned char temppassdigit = descrambledpass[x]<<5;
		for(i = 0;i < 3;i++){ //this iterates the below 3 times to extract all 3 bits
			while(bittable[(y*12)+x] == 0)y++;
			if(temppassdigit & 0x80){
				decodeddata[y] = (bittable[(y*12)+x] | decodeddata[y]);
			}
			temppassdigit = temppassdigit<<1;
			y++;
		}
	}
    //If all went well we should have our decoded data at this point
	//Now we must do some processing on it, and verify it, this is based on $00:F1B4
	decodeddata[0] = (decodeddata[0] ^ ((decodeddata[3] & 0xf0) | ((decodeddata[3] & 0xf0) >> 4)));
	decodeddata[1] = (decodeddata[1] ^ ((decodeddata[3] & 0xf0) | ((decodeddata[3] & 0xf0) >> 4)));
	decodeddata[2] = (decodeddata[2] ^ ((decodeddata[3] & 0xf0) | ((decodeddata[3] & 0xf0) >> 4)));

	//Now here comes some validity checks, This is based at $00:F1D2
	bool passwordok = false;
	if(decodeddata[3] & 0x10){
		passwordok = true;
	}
	else{ 
		if((decodeddata[0] | decodeddata[1] | decodeddata[2]) == 0){
			passwordok = true;
		}
	}
	//this check is at $00:F1EB
	if(decodeddata[4] != ((((decodeddata[0] ^ decodeddata[1] ^ decodeddata[2] ^decodeddata[3])>>4) 
            ^ (decodeddata[0] ^ decodeddata[1] ^ decodeddata[2] ^decodeddata[3])) & 0x0f))passwordok = false;

	//here comes another check found at $00:F201
	x = 0;
	if(decodeddata[0] & 0x01)x++;
	if(decodeddata[0] & 0x02)x++;
	if(decodeddata[0] & 0x04)x++;
	if(decodeddata[0] & 0x08)x++;
	if(decodeddata[0] & 0x10)x++;
	if(decodeddata[0] & 0x20)x++;
	if(decodeddata[0] & 0x40)x++;
	if(decodeddata[0] & 0x80)x++;
	if((x ^ decodeddata[3]>>3) & 0x01)passwordok = false;

	//This one here is found at $01:F213
	x = 0;
	if(decodeddata[1] & 0x01)x++;
	if(decodeddata[1] & 0x02)x++;
	if(decodeddata[1] & 0x04)x++;
	if(decodeddata[1] & 0x08)x++;
	if(decodeddata[1] & 0x10)x++;
	if(decodeddata[1] & 0x20)x++;
	if(decodeddata[1] & 0x40)x++;
	if(decodeddata[1] & 0x80)x++;
	if((x ^ decodeddata[3]>>2) & 0x01)passwordok = false;

	//This one here is found at $01:F224
	x = 0;
	if(decodeddata[2] & 0x01)x++;
	if(decodeddata[2] & 0x02)x++;
	if(decodeddata[2] & 0x04)x++;
	if(decodeddata[2] & 0x08)x++;
	if(decodeddata[2] & 0x10)x++;
	if(decodeddata[2] & 0x20)x++;
	if(decodeddata[2] & 0x40)x++;
	if(decodeddata[2] & 0x80)x++;
	if((x ^ decodeddata[3]>>1) & 0x01)passwordok = false;
	
	//before we return decodeddata lets fix something up
	decodeddata[3] = ((decodeddata[3]&0xF0)>>3 | decodeddata[3]&1);

	if(passwordok)return decodeddata;
	else return NULL;
}

//there are 29 bits of decoded password data, when passing to this function
//you should have them in decodeddata[0],[1], and [2] the other 5 bits are special
//and should be stored in the lowest 5 bits of decodeddata[3]
//in other words, if decodeddata[3] is ANDed with 0x1F if shouldn't affect it
unsigned char *GeneratePass(unsigned char decodeddata[5]){
	int x,y,i;
	static unsigned char password[12]; //we will store the password here at the end

	//fix decodeddata[3] to how we need it
	decodeddata[3] = (decodeddata[3] << 3);
    decodeddata[3] = (decodeddata[3] | ((decodeddata[3]>>3)&1))&0xF1;

	//if an important bit in decodeddata[3] isn't set then clear the rest
	if(!(decodeddata[3] & 0x10)) {
		decodeddata[0] = 0;
		decodeddata[1] = 0;
		decodeddata[2] = 0;
	}

    //now we do some work on decodeddata
	x = 0;
	if(decodeddata[2] & 0x01)x++;
	if(decodeddata[2] & 0x02)x++;
	if(decodeddata[2] & 0x04)x++;
	if(decodeddata[2] & 0x08)x++;
	if(decodeddata[2] & 0x10)x++;
	if(decodeddata[2] & 0x20)x++;
	if(decodeddata[2] & 0x40)x++;
	if(decodeddata[2] & 0x80)x++;
    decodeddata[3] ^= (x&1)<<1;

	x = 0;
	if(decodeddata[1] & 0x01)x++;
	if(decodeddata[1] & 0x02)x++;
	if(decodeddata[1] & 0x04)x++;
	if(decodeddata[1] & 0x08)x++;
	if(decodeddata[1] & 0x10)x++;
	if(decodeddata[1] & 0x20)x++;
	if(decodeddata[1] & 0x40)x++;
	if(decodeddata[1] & 0x80)x++;
    decodeddata[3] ^= (x&1)<<2;

	x = 0;
	if(decodeddata[0] & 0x01)x++;
	if(decodeddata[0] & 0x02)x++;
	if(decodeddata[0] & 0x04)x++;
	if(decodeddata[0] & 0x08)x++;
	if(decodeddata[0] & 0x10)x++;
	if(decodeddata[0] & 0x20)x++;
	if(decodeddata[0] & 0x40)x++;
	if(decodeddata[0] & 0x80)x++;
    decodeddata[3] ^= (x&1)<<3;

	//this here encrypts it the same way the game does at 00:F103
	decodeddata[4] = (((decodeddata[0] ^ decodeddata[1] ^ decodeddata[2] ^ decodeddata[3]) ^
		((decodeddata[0] ^ decodeddata[1] ^ decodeddata[2] ^ decodeddata[3])>>4)) & 0x0f);

	//more encrypting at 00:F117
	decodeddata[0] = decodeddata[0] ^ ((decodeddata[3]& 0xF0) | (decodeddata[3]>>4));
	decodeddata[1] = decodeddata[1] ^ ((decodeddata[3]& 0xF0) | (decodeddata[3]>>4));
	decodeddata[2] = decodeddata[2] ^ ((decodeddata[3]& 0xF0) | (decodeddata[3]>>4));

	//now its time to get dirty and do a nasty looking loop that extracts a password
	//from the bits
	for(i = 0;i < 12;i++){
		unsigned char tempchardata = 0;
		
		for(y = 0;y < 5;y++){
			if(bittable[i+(y*12)]){
				if(bittable[i+(y*12)] & decodeddata[y])tempchardata = (tempchardata<<1) | 1;
				else tempchardata = tempchardata<<1;
			}
		}
		password[i] = passwordcrambledata[(i*8)+(tempchardata)];
	}
	return password;
}

//GenerateRandomPass()
//Function to Generate a random password by randomly testing untill something works
//it fills the global variable g_currentdiaplyedpass with the password when it returns
//as of version 1.0 this function is no longer used. But I left it here anyway
void GenerateRandomPass(){
   int i = 0;

	   do{
	   g_currentdisplayedpass[0] = (rand()%8);
	   g_currentdisplayedpass[1] = (rand()%8);
	   g_currentdisplayedpass[2] = (rand()%8);
	   g_currentdisplayedpass[3] = (rand()%8);
	   g_currentdisplayedpass[4] = (rand()%8);
	   g_currentdisplayedpass[5] = (rand()%8);
	   g_currentdisplayedpass[6] = (rand()%8);
	   g_currentdisplayedpass[7] = (rand()%8);
	   g_currentdisplayedpass[8] = (rand()%8);
	   g_currentdisplayedpass[9] = (rand()%8);
	   g_currentdisplayedpass[10] = (rand()%8);
	   g_currentdisplayedpass[11] = (rand()%8);
	   i++;
	   }while(VerifyPass(g_currentdisplayedpass) == NULL);
   
   return;
}

//this is called once before any calls to CreatePasswordBitmap() to init any bitmaps it uses
void InitializeBitmaps(){
   g_blankscreenbitmap = new Bitmap(g_hInstance,L"blankpasswordscreen");
   g_numberbitmaps[0] = new Bitmap(g_hInstance,L"passnumber1");
   g_numberbitmaps[1] = new Bitmap(g_hInstance,L"passnumber2");
   g_numberbitmaps[2] = new Bitmap(g_hInstance,L"passnumber3");
   g_numberbitmaps[3] = new Bitmap(g_hInstance,L"passnumber4");
   g_numberbitmaps[4] = new Bitmap(g_hInstance,L"passnumber5");
   g_numberbitmaps[5] = new Bitmap(g_hInstance,L"passnumber6");
   g_numberbitmaps[6] = new Bitmap(g_hInstance,L"passnumber7");
   g_numberbitmaps[7] = new Bitmap(g_hInstance,L"passnumber8");

   return;
}

//this is called before the program ends to destroy what InitializeBitmaps() created
void DestroyBitmaps(){
   delete g_blankscreenbitmap;
   for(int i = 0;i < 8;i++)delete g_numberbitmaps[i];

   return;
}

//This creates a new bitmap with the password values filled in based on the password data string
//remember, the returned bitmap must be destroyed
Bitmap *CreatePasswordBitmap(unsigned char *password){
   //First we need the temporary bitmap that will be working on. I initialize it with size
   //information from the empty password screen
   Bitmap *buffbitmap = new Bitmap(g_blankscreenbitmap->GetWidth(),
   g_blankscreenbitmap->GetHeight(),g_blankscreenbitmap->GetPixelFormat());

   //next we need a graphics object to that bitmap so we can work on it
   Graphics graphicsbuff(buffbitmap);

   //start by drawing our blank password screen bitmap on it
   graphicsbuff.DrawImage(g_blankscreenbitmap,0,0);

   //This loop draws each number to the correct x and y position
   for(int i = 0;i < 12;i++)
      graphicsbuff.DrawImage(g_numberbitmaps[password[i]],41+((i%4)*48),23+((i/4)*48));

   return buffbitmap;
}

//This function checks the state of all the check boxes and shows the password based on them
void GenerateAndDisplayPasswordFromCheckBoxes(HWND hWnd){
	unsigned char dummydata[5];
	unsigned char *temppass;
	HDC          hdc;
	dummydata[0] = 0;
	dummydata[1] = 0;
	dummydata[2] = 0;
	dummydata[3] = 0;
	if(IsDlgButtonChecked(hWnd,IDC_BIT1CHECK) == BST_CHECKED)dummydata[0] |= 0x80;
	if(IsDlgButtonChecked(hWnd,IDC_BIT2CHECK) == BST_CHECKED)dummydata[0] |= 0x40;
	if(IsDlgButtonChecked(hWnd,IDC_BIT3CHECK) == BST_CHECKED)dummydata[0] |= 0x20;
	if(IsDlgButtonChecked(hWnd,IDC_BIT4CHECK) == BST_CHECKED)dummydata[0] |= 0x10;
	if(IsDlgButtonChecked(hWnd,IDC_BIT5CHECK) == BST_CHECKED)dummydata[0] |= 0x08;
	if(IsDlgButtonChecked(hWnd,IDC_BIT6CHECK) == BST_CHECKED)dummydata[0] |= 0x04;
	if(IsDlgButtonChecked(hWnd,IDC_BIT7CHECK) == BST_CHECKED)dummydata[0] |= 0x02;
	if(IsDlgButtonChecked(hWnd,IDC_BIT8CHECK) == BST_CHECKED)dummydata[0] |= 0x01;
	if(IsDlgButtonChecked(hWnd,IDC_BIT9CHECK) == BST_CHECKED)dummydata[1] |= 0x80;
	if(IsDlgButtonChecked(hWnd,IDC_BIT10CHECK) == BST_CHECKED)dummydata[1] |= 0x40;
	if(IsDlgButtonChecked(hWnd,IDC_BIT11CHECK) == BST_CHECKED)dummydata[1] |= 0x20;
	if(IsDlgButtonChecked(hWnd,IDC_BIT12CHECK) == BST_CHECKED)dummydata[1] |= 0x10;
	if(IsDlgButtonChecked(hWnd,IDC_BIT13CHECK) == BST_CHECKED)dummydata[1] |= 0x08;
	if(IsDlgButtonChecked(hWnd,IDC_BIT14CHECK) == BST_CHECKED)dummydata[1] |= 0x04;
	if(IsDlgButtonChecked(hWnd,IDC_BIT15CHECK) == BST_CHECKED)dummydata[1] |= 0x02;
	if(IsDlgButtonChecked(hWnd,IDC_BIT16CHECK) == BST_CHECKED)dummydata[1] |= 0x01;
	if(IsDlgButtonChecked(hWnd,IDC_BIT17CHECK) == BST_CHECKED)dummydata[2] |= 0x80;
	if(IsDlgButtonChecked(hWnd,IDC_BIT18CHECK) == BST_CHECKED)dummydata[2] |= 0x40;
	if(IsDlgButtonChecked(hWnd,IDC_BIT19CHECK) == BST_CHECKED)dummydata[2] |= 0x20;
	if(IsDlgButtonChecked(hWnd,IDC_BIT20CHECK) == BST_CHECKED)dummydata[2] |= 0x10;
	if(IsDlgButtonChecked(hWnd,IDC_BIT21CHECK) == BST_CHECKED)dummydata[2] |= 0x08;
	if(IsDlgButtonChecked(hWnd,IDC_BIT22CHECK) == BST_CHECKED)dummydata[2] |= 0x04;
	if(IsDlgButtonChecked(hWnd,IDC_BIT23CHECK) == BST_CHECKED)dummydata[2] |= 0x02;
	if(IsDlgButtonChecked(hWnd,IDC_BIT24CHECK) == BST_CHECKED)dummydata[2] |= 0x01;
	if(IsDlgButtonChecked(hWnd,IDC_BIT25CHECK) == BST_CHECKED)dummydata[3] |= 0x10;
	if(IsDlgButtonChecked(hWnd,IDC_BIT26CHECK) == BST_CHECKED)dummydata[3] |= 0x08;
	if(IsDlgButtonChecked(hWnd,IDC_BIT27CHECK) == BST_CHECKED)dummydata[3] |= 0x04;
	if(IsDlgButtonChecked(hWnd,IDC_BIT28CHECK) == BST_CHECKED)dummydata[3] |= 0x02;
	if(IsDlgButtonChecked(hWnd,IDC_BIT29CHECK) == BST_CHECKED)dummydata[3] |= 0x01;

	temppass = GeneratePass(dummydata);
	for(int i = 0;i < 12;i++)g_currentdisplayedpass[i] = temppass[i];
			
	hdc = GetDC(hWnd);
	DisplayPassword(hdc);
	ReleaseDC(hWnd,hdc);
	return;
}

//This function sets the check boxes based on the password
void SetCheckBoxesFromPassword(HWND hWnd){
	unsigned char *dummydata;
	
	//retrieve data
	dummydata = VerifyPass(g_currentdisplayedpass);

	//if it isn't valid then make an error and return
	if(dummydata == NULL){
		MessageBox(hWnd,"Password is Invalid","Error",IDOK);
		return;
	}

	if(dummydata[0] & 0x80)CheckDlgButton(hWnd,IDC_BIT1CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT1CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x40)CheckDlgButton(hWnd,IDC_BIT2CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT2CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x20)CheckDlgButton(hWnd,IDC_BIT3CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT3CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x10)CheckDlgButton(hWnd,IDC_BIT4CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT4CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x08)CheckDlgButton(hWnd,IDC_BIT5CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT5CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x04)CheckDlgButton(hWnd,IDC_BIT6CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT6CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x02)CheckDlgButton(hWnd,IDC_BIT7CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT7CHECK,BST_UNCHECKED);
	if(dummydata[0] & 0x01)CheckDlgButton(hWnd,IDC_BIT8CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT8CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x80)CheckDlgButton(hWnd,IDC_BIT9CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT9CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x40)CheckDlgButton(hWnd,IDC_BIT10CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT10CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x20)CheckDlgButton(hWnd,IDC_BIT11CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT11CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x10)CheckDlgButton(hWnd,IDC_BIT12CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT12CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x08)CheckDlgButton(hWnd,IDC_BIT13CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT13CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x04)CheckDlgButton(hWnd,IDC_BIT14CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT14CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x02)CheckDlgButton(hWnd,IDC_BIT15CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT15CHECK,BST_UNCHECKED);
	if(dummydata[1] & 0x01)CheckDlgButton(hWnd,IDC_BIT16CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT16CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x80)CheckDlgButton(hWnd,IDC_BIT17CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT17CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x40)CheckDlgButton(hWnd,IDC_BIT18CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT18CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x20)CheckDlgButton(hWnd,IDC_BIT19CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT19CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x10)CheckDlgButton(hWnd,IDC_BIT20CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT20CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x08)CheckDlgButton(hWnd,IDC_BIT21CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT21CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x04)CheckDlgButton(hWnd,IDC_BIT22CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT22CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x02)CheckDlgButton(hWnd,IDC_BIT23CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT23CHECK,BST_UNCHECKED);
	if(dummydata[2] & 0x01)CheckDlgButton(hWnd,IDC_BIT24CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT24CHECK,BST_UNCHECKED);
	if(dummydata[3] & 0x10)CheckDlgButton(hWnd,IDC_BIT25CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT25CHECK,BST_UNCHECKED);
	if(dummydata[3] & 0x08)CheckDlgButton(hWnd,IDC_BIT26CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT26CHECK,BST_UNCHECKED);
	if(dummydata[3] & 0x04)CheckDlgButton(hWnd,IDC_BIT27CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT27CHECK,BST_UNCHECKED);
	if(dummydata[3] & 0x02)CheckDlgButton(hWnd,IDC_BIT28CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT28CHECK,BST_UNCHECKED);
	if(dummydata[3] & 0x01)CheckDlgButton(hWnd,IDC_BIT29CHECK,BST_CHECKED); else CheckDlgButton(hWnd,IDC_BIT29CHECK,BST_UNCHECKED);
	return;
}

//This enables or disables checkboxes 1-24 based on if bEnable is TRUE or FALSE
void EnableCheckBoxes(HWND hWnd,BOOL bEnable){

	CheckDlgButton(hWnd,IDC_BIT1CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT2CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT3CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT4CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT5CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT6CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT7CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT8CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT9CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT10CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT11CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT12CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT13CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT14CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT15CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT16CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT17CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT18CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT19CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT20CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT21CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT22CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT23CHECK,BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_BIT24CHECK,BST_UNCHECKED);

	EnableWindow(GetDlgItem(hWnd,IDC_BIT1CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT2CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT3CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT4CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT5CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT6CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT7CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT8CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT9CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT10CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT11CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT12CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT13CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT14CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT15CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT16CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT17CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT18CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT19CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT20CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT21CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT22CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT23CHECK),bEnable);
	EnableWindow(GetDlgItem(hWnd,IDC_BIT24CHECK),bEnable);
return;
}